#
# See the file "LICENSE" for the full license governing this code.
#
$:.unshift File.dirname(__FILE__)
$:.unshift File.join(File.dirname(__FILE__), '../../../easy_type/lib')
module Puppet
  Type.newtype(:wls_managedserver) do
    def self.module_name;'wls_config' end

    desc <<-EOD
    control a managed server or cluster state like running,stop,restart

    Type for managed server control like start, running, abort and stop a managed server or a cluster. 
    Also supports `subscribe` with `refreshonly` when you only want to execute the restart when something
    has changed.

    Here is an example on how you should use this:

        wls_managedserver{'JMSServer1_Wls1036:':
          ensure                    => 'running',   #running|start|abort|stop
          target                    => 'Server', #Server|Cluster
          server_name               => 'JMSServer1',
          domain_name               => hiera('domain_name'),
          os_user                   => hiera('wls_os_user'),
          weblogic_home_dir         => hiera('wls_weblogic_home_dir'),
          weblogic_user             => hiera('wls_weblogic_user'),
          weblogic_password         => hiera('domain_wls_password'),
          jdk_home_dir              => hiera('wls_jdk_home_dir'),
          adminserver_address       => hiera('domain_adminserver_address'),
          adminserver_port          => hiera('domain_adminserver_port'),
        }

    Here is an example with `refreshonly`. In this example, the restart will only be executed
    when the domain `Wls1036`  is changed. A restart will be executed.

        wls_managedserver{'JMSServer1_Wls1036':
          ensure                    => 'restart',   #running|start|abort|stop
          target                    => 'Server',    #Server|Cluster
          server_name               => 'JMSServer1',
          domain_name               => hiera('domain_name'),
          os_user                   => hiera('wls_os_user'),
          weblogic_home_dir         => hiera('wls_weblogic_home_dir'),
          weblogic_user             => hiera('wls_weblogic_user'),
          weblogic_password         => hiera('domain_wls_password'),
          jdk_home_dir              => hiera('wls_jdk_home_dir'),
          adminserver_address       => hiera('domain_adminserver_address'),
          adminserver_port          => hiera('domain_adminserver_port'),
          refreshonly               => true,
          subscribe                 => Wls_domain['Wls1036'],
        }

    EOD

    newproperty(:ensure) do

    desc <<-EOD
    Whether to do something.
    EOD

      newvalue(:start, :event => :managedserver_running) do
        unless :refreshonly == true
          provider.start
        end
      end

      newvalue(:stop, :event => :managedserver_running) do
        unless :refreshonly == true
          provider.stop
        end
      end

      aliasvalue(:running, :start)
      aliasvalue(:abort, :stop)

      def retrieve
        provider.status
      end

      def sync
        event = super()
        # rubocop:disable all
        if property = @resource.property(:enable)
          val = property.retrieve
          property.sync unless property.safe_insync?(val)
        end
        # rubocop:enable all
        event
      end
    end

    newparam(:name) do
      desc <<-EOT
        The title.
      EOT
      isnamevar
    end

    newparam(:target) do
      desc <<-EOT
        The weblogic type.
      EOT
    end

    newparam(:server_name) do
      desc <<-EOT
        The adminserver name.
      EOT
    end

    newparam(:domain_name) do
      desc <<-EOT
        The weblogic domain name.
      EOT
    end

    newparam(:os_user) do
      desc <<-EOT
        The weblogic operating system user.
      EOT
    end

    newparam(:weblogic_home_dir) do
      desc <<-EOT
        The weblogic home folder.
      EOT
    end

    newparam(:weblogic_user) do
      desc <<-EOT
        The weblogic user.
      EOT
    end

    newparam(:weblogic_password) do
      desc <<-EOT
        The weblogic user password.
      EOT
    end

    newparam(:jdk_home_dir) do
      desc <<-EOT
        The jdk home dir.
      EOT
    end

    newparam(:adminserver_address) do
      desc <<-EOT
        The adminserver address.
      EOT
    end

    newparam(:adminserver_port) do
      desc <<-EOT
        The adminserver port.
      EOT
    end

    newparam(:refreshonly) do
      desc <<-EOT
        The command should only be run as a
        refresh mechanism for when a dependent object is changed.
      EOT

      newvalues(:true, :false)

      defaultto :false
    end

    def refresh
      Puppet.info 'wls_managedserver refresh'
      provider.restart
    end

  end
end
